#!/usr/bin/perl -T
use warnings;
use strict;

use lib qw/ t lib /;
use Test::More tests => 1;

BEGIN { use_ok( 'CGI::ValidOp::Test' )}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# vim:ft=perl
